// ConflictEditorDoc.cpp : implementation of the ConflictEditorDoc class
//

#include "stdafx.h"
#include "CvsConflictEditor.h"
#include "ConflictBlock.h"
#include "ConflictEditorDoc.h"
#include "ConflictEditorUpdateContext.h"
#include "ConflictEditorView.h"
#include "ConflictFileLoader.h"
#include "ConflictList.h"
#include "ConflictTextBuffer.h"
#include "ConflictTextBufferBuilder.h"


/* Note:
 - addMergedRevisionToTitle(): setting the document title does not work ?!?
 */


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif



IMPLEMENT_DYNCREATE(ConflictEditorDoc, CDocument)

BEGIN_MESSAGE_MAP(ConflictEditorDoc, CDocument)
	//{{AFX_MSG_MAP(ConflictEditorDoc)
	ON_COMMAND(ID_NEXT_CONFLICT, OnNextConflict)
	ON_UPDATE_COMMAND_UI(ID_NEXT_CONFLICT, OnUpdateNextConflict)
	ON_COMMAND(ID_PREVIOUS_CONFLICT, OnPreviousConflict)
	ON_UPDATE_COMMAND_UI(ID_PREVIOUS_CONFLICT, OnUpdatePreviousConflict)
	ON_COMMAND(ID_FIRST_CONFLICT, OnFirstConflict)
	ON_UPDATE_COMMAND_UI(ID_FIRST_CONFLICT, OnUpdateFirstConflict)
	ON_COMMAND(ID_LAST_CONFLICT, OnLastConflict)
	ON_UPDATE_COMMAND_UI(ID_LAST_CONFLICT, OnUpdateLastConflict)
	ON_COMMAND(ID_CURRENT_CONFLICT, OnCurrentConflict)
	ON_UPDATE_COMMAND_UI(ID_CURRENT_CONFLICT, OnUpdateCurrentConflict)
	ON_COMMAND(ID_INSERT_MERGED_BEFORE_TARGET, OnInsertMergedBeforeTarget)
	ON_UPDATE_COMMAND_UI(ID_INSERT_MERGED_BEFORE_TARGET, OnUpdateInsertMergedBeforeTarget)
	ON_COMMAND(ID_INSERT_MERGED_AFTER_TARGET, OnInsertMergedAfterTarget)
	ON_UPDATE_COMMAND_UI(ID_INSERT_MERGED_AFTER_TARGET, OnUpdateInsertMergedAfterTarget)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()



ConflictEditorDoc::ConflictEditorDoc() :
  m_nCurrentConflictIndex( 0 )
{
  m_pTargetTextBuffer = new ConflictTextBuffer( this );
  m_pMergedTextBuffer = new ConflictTextBuffer( this );
}

ConflictEditorDoc::~ConflictEditorDoc()
{
  delete m_pTargetTextBuffer;
  delete m_pMergedTextBuffer;
}


void 
ConflictEditorDoc::DeleteContents() 
{
  m_pTargetTextBuffer->FreeAll();
  m_pMergedTextBuffer->FreeAll();
	CDocument::DeleteContents();
}


BOOL 
ConflictEditorDoc::OnNewDocument()
{
	if (!CDocument::OnNewDocument())
		return FALSE;

	// TODO: add reinitialization code here
	// (SDI documents will reuse this document)
  m_pTargetTextBuffer->InitNew();
  m_pMergedTextBuffer->InitNew();
  m_pMergedTextBuffer->SetReadOnly();

	return TRUE;
}



void 
ConflictEditorDoc::Serialize(CArchive& ar)
{
	if (ar.IsStoring())
	{
		// TODO: add storing code here
	}
	else
	{
		// TODO: add loading code here
	}
}


#ifdef _DEBUG
void 
ConflictEditorDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void 
ConflictEditorDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG


void 
ConflictEditorDoc::textBufferModified( ConflictTextBuffer *buffer,
                                       bool bIsModified )
{
  SetModifiedFlag( bIsModified );
}


ConflictTextBuffer *
ConflictEditorDoc::targetTextBuffer()
{
  ASSERT( m_pTargetTextBuffer );
  return m_pTargetTextBuffer;
}


ConflictTextBuffer *
ConflictEditorDoc::mergedTextBuffer()
{
  ASSERT( m_pMergedTextBuffer );
  return m_pMergedTextBuffer;
}


BOOL 
ConflictEditorDoc::OnOpenDocument(LPCTSTR lpszPathName) 
{
	if (!CDocument::OnOpenDocument(lpszPathName))
		return FALSE;

  ConflictTextBufferBuilder targetBuilder( m_pTargetTextBuffer );
  ConflictTextBufferBuilder mergedBuilder( m_pMergedTextBuffer );
  ConflictFileLoader loader( lpszPathName,       
                             &targetBuilder, 
                             &mergedBuilder );

  if ( !loader.loadFile() )
    return FALSE;

  m_pMergedTextBuffer->SetReadOnly();
  if ( !loader.mergedRevision().empty() )
    addMergedRevisionToTitle( loader.mergedRevision() );
  
  return TRUE;
}


BOOL 
ConflictEditorDoc::OnSaveDocument(LPCTSTR lpszPathName) 
{
  return m_pTargetTextBuffer->SaveToFile( lpszPathName, 
                                          m_pMergedTextBuffer->GetCRLFMode() );
}


void 
ConflictEditorDoc::scrollViewsToCurrentConflict()
{
  ConflictEditorUpdateContext context( m_nCurrentConflictIndex );
  UpdateAllViews( NULL, 0, &context );
}


int 
ConflictEditorDoc::conflictCount()
{
  return m_pTargetTextBuffer->conflicts()->conflictCount();
}


void 
ConflictEditorDoc::OnNextConflict() 
{
  ++m_nCurrentConflictIndex;
  scrollViewsToCurrentConflict();
}


void 
ConflictEditorDoc::OnUpdateNextConflict(CCmdUI* pCmdUI) 
{
  pCmdUI->Enable( conflictCount()-1 > m_nCurrentConflictIndex );
}


void 
ConflictEditorDoc::OnPreviousConflict() 
{
  --m_nCurrentConflictIndex;
  scrollViewsToCurrentConflict();
}


void 
ConflictEditorDoc::OnUpdatePreviousConflict(CCmdUI* pCmdUI) 
{
  pCmdUI->Enable( m_nCurrentConflictIndex > 0 );
}


void 
ConflictEditorDoc::OnFirstConflict() 
{
  if ( conflictCount() > 0 )
  {
    m_nCurrentConflictIndex = 0;
    scrollViewsToCurrentConflict();
  }
}


void 
ConflictEditorDoc::OnUpdateFirstConflict(CCmdUI* pCmdUI) 
{
  pCmdUI->Enable( conflictCount() > 0 );
}


void 
ConflictEditorDoc::OnLastConflict() 
{
  m_nCurrentConflictIndex = conflictCount() -1;
  scrollViewsToCurrentConflict();
}


void 
ConflictEditorDoc::OnUpdateLastConflict(CCmdUI* pCmdUI) 
{
  pCmdUI->Enable( conflictCount() > 0 );
}


void 
ConflictEditorDoc::OnCurrentConflict() 
{
  scrollViewsToCurrentConflict();
}


void 
ConflictEditorDoc::OnUpdateCurrentConflict(CCmdUI* pCmdUI) 
{
  pCmdUI->Enable( conflictCount() > 0 );
}


int
ConflictEditorDoc::currentConflictIndex()
{
  return m_nCurrentConflictIndex;
}


void 
ConflictEditorDoc::addMergedRevisionToTitle( std::string mergedRevision )
{
  CString title( GetTitle() );
  title += " merged with revision ";
  title += mergedRevision.c_str();
  SetTitle( title );
}


ConflictEditorView *
ConflictEditorDoc::targetView()
{
  POSITION viewIterator = GetFirstViewPosition();
  ConflictEditorView *pTargetView;
  do
  {
    pTargetView = dynamic_cast<ConflictEditorView *>( GetNextView( viewIterator ) );
  }
  while ( pTargetView == NULL  &&  viewIterator != NULL );
  ASSERT( pTargetView != NULL ); // failed: used this method when not allowed to.
  return pTargetView;
}


ConflictBlock *
ConflictEditorDoc::targetConflictAt( int nConflictIndex )
{
  return m_pTargetTextBuffer->conflicts()->conflictAt( nConflictIndex );
}


ConflictBlock *
ConflictEditorDoc::mergedConflictAt( int nConflictIndex )
{
  return m_pMergedTextBuffer->conflicts()->conflictAt( nConflictIndex );
}


void 
ConflictEditorDoc::OnInsertMergedBeforeTarget() 
{
  ConflictBlock *pMergedConflict = mergedConflictAt( m_nCurrentConflictIndex );
  ConflictBlock *pTargetConflict = targetConflictAt( m_nCurrentConflictIndex );

  for ( int nLineIndex =0; nLineIndex < pMergedConflict->numberOfLines(); ++nLineIndex )
  {
    int nUnusedEndLine;
    int nUnusedEndChar;

    m_pTargetTextBuffer->InsertText( targetView(), 
        pTargetConflict->startingLineNumber() + nLineIndex, 0,
        (pMergedConflict->conflictingTextLineAt( nLineIndex )+"\r\n").c_str(),
        nUnusedEndLine, nUnusedEndChar,
        CE_ACTION_PASTE );
  }
}


void 
ConflictEditorDoc::OnUpdateInsertMergedBeforeTarget(CCmdUI* pCmdUI) 
{
  pCmdUI->Enable( conflictCount() > 0 );
}


void 
ConflictEditorDoc::OnInsertMergedAfterTarget() 
{
  ConflictBlock *pMergedConflict = mergedConflictAt( m_nCurrentConflictIndex );
  ConflictBlock *pTargetConflict = targetConflictAt( m_nCurrentConflictIndex );

  int nInsertionLineIndex = pTargetConflict->endLineNumber();
  for ( int nLineIndex =0; nLineIndex < pMergedConflict->numberOfLines(); ++nLineIndex )
  {
    int nUnusedEndLine;
    int nUnusedEndChar;

    m_pTargetTextBuffer->InsertText( targetView(), 
        nInsertionLineIndex + nLineIndex, m_pTargetTextBuffer->GetLineLength(nInsertionLineIndex + nLineIndex),
        ("\r\n"+pMergedConflict->conflictingTextLineAt( nLineIndex )).c_str(),
        nUnusedEndLine, nUnusedEndChar,
        CE_ACTION_PASTE );
  }
}


void 
ConflictEditorDoc::OnUpdateInsertMergedAfterTarget(CCmdUI* pCmdUI) 
{
  pCmdUI->Enable( conflictCount() > 0 );
}
