// ConflictEditorView.cpp : implementation of the ConflictEditorView class
//

#include "stdafx.h"
#include "CvsConflictEditor.h"
#include "CrystalEdit/CCrystalTextBuffer.h"
#include "ConflictBlock.h"
#include "ConflictEditorDoc.h"
#include "ConflictEditorUpdateContext.h"
#include "ConflictEditorView.h"
#include "ConflictList.h"
#include "ConflictMarginDrawer.h"
#include "ConflictTextBuffer.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif



IMPLEMENT_DYNCREATE(ConflictEditorView, CCrystalEditViewEx)

BEGIN_MESSAGE_MAP(ConflictEditorView, CCrystalEditViewEx)
	//{{AFX_MSG_MAP(ConflictEditorView)
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CCrystalEditViewEx::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CCrystalEditViewEx::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CCrystalEditViewEx::OnFilePrintPreview)
END_MESSAGE_MAP()



ConflictEditorView::ConflictEditorView() :
  m_pParser( new CCrystalParser() ),
  m_bIsTargetView( true )
{
  m_pMarginDrawer = new ConflictMarginDrawer();
  SetParser( m_pParser );
}


ConflictEditorView::~ConflictEditorView()
{
  DetachFromBuffer();
  delete m_pParser;
  delete m_pMarginDrawer;
}


BOOL 
ConflictEditorView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

  cs.style |= WS_VSCROLL;

	return CCrystalEditViewEx::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// ConflictEditorView drawing
/*
void 
ConflictEditorView::OnDraw(CDC* pDC)
{
	ConflictEditorDoc* pDoc = document();
	ASSERT_VALID(pDoc);
	// TODO: add draw code for native data here
}
*/

/////////////////////////////////////////////////////////////////////////////
// ConflictEditorView printing

BOOL 
ConflictEditorView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}


void 
ConflictEditorView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add extra initialization before printing
}


void 
ConflictEditorView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}

/////////////////////////////////////////////////////////////////////////////
// ConflictEditorView diagnostics

#ifdef _DEBUG
void 
ConflictEditorView::AssertValid() const
{
	CCrystalEditViewEx::AssertValid();
}


void 
ConflictEditorView::Dump(CDumpContext& dc) const
{
	CCrystalEditViewEx::Dump(dc);
}
#endif //_DEBUG


ConflictEditorDoc* 
ConflictEditorView::document() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(ConflictEditorDoc)));
	return (ConflictEditorDoc*)m_pDocument;
}


CCrystalTextBuffer *
ConflictEditorView::LocateTextBuffer()
{
  return textBuffer();
}


void 
ConflictEditorView::OnInitialUpdate() 
{
	CCrystalEditViewEx::OnInitialUpdate();

	//	Initialize LOGFONT structure
	memset(&m_font, 0, sizeof(m_font));
	m_font.lfWeight = FW_NORMAL;
	m_font.lfCharSet = ANSI_CHARSET;
	m_font.lfOutPrecision = OUT_DEFAULT_PRECIS;
	m_font.lfClipPrecision = CLIP_DEFAULT_PRECIS;
	m_font.lfQuality = DEFAULT_QUALITY;
	m_font.lfPitchAndFamily = DEFAULT_PITCH | FF_DONTCARE;
	strcpy(m_font.lfFaceName, "Courier");
  
  AttachToBuffer( LocateTextBuffer() );   //document()->textBuffer() );
	SetFont( m_font );
  SetSelectionMargin( TRUE );
  SetDisableBSAtSOL( FALSE );
}


void 
ConflictEditorView::DrawMargin( CDC * pdc, 
                                const CRect & rect, 
                                int nLineIndex )
{
  CCrystalEditViewEx::DrawMargin( pdc, rect, nLineIndex );

  m_pMarginDrawer->DrawMargin( pdc, 
                               rect, 
                               nLineIndex, 
                               textBuffer(), 
                               document()->currentConflictIndex() );
}


void 
ConflictEditorView::SetTargetView()
{
  m_bIsTargetView = true;
}


void 
ConflictEditorView::SetMergedView()
{
  m_bIsTargetView = false;
}


ConflictTextBuffer *
ConflictEditorView::textBuffer()
{
  if ( m_bIsTargetView )
    return document()->targetTextBuffer();
  return document()->mergedTextBuffer();
}


ConflictList *
ConflictEditorView::conflicts()
{
  return textBuffer()->conflicts();
}


void 
ConflictEditorView::OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint) 
{
  ConflictEditorUpdateContext *pContext = 
      dynamic_cast<ConflictEditorUpdateContext *>( pHint );
  if ( pContext == NULL )
    CCrystalEditViewEx::OnUpdate( pSender, lHint, pHint );
  else
    updateView( pContext );
}


void
ConflictEditorView::updateView( ConflictEditorUpdateContext *pContext )
{
  int nFocusConflictNumber = pContext->focusConflictNumber();
  ConflictBlock *pConflict = conflicts()->conflictAt( nFocusConflictNumber );
  ScrollToLine( pConflict->startingLineNumber() );
}
