// //////////////////////////////////////////////////////////////////////////
// Implementation file ConflictList.cpp for class ConflictList
// (c)Copyright 2000, Baptiste Lepilleur.
// Created: 2001/06/17
// //////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "ConflictList.h"
#include "ConflictBlock.h"


ConflictList::ConflictList()
{
}


ConflictList::~ConflictList()
{
}


int 
ConflictList::conflictBlockStatusFor( int nLineIndex,
                                      int nSelectedConflictIndex )
{
  int nBlockIndex = findConflictBlockAt( nLineIndex );
  if ( nBlockIndex >= 0 )
  {
    int nLineStatus = m_conflicts.at( nBlockIndex )->lineStatus( nLineIndex );
    if ( nSelectedConflictIndex == nBlockIndex )
      nLineStatus |= ConflictBlock::lineSelectedConflictBlock;
    return nLineStatus;
  }

  return ConflictBlock::lineOutsideConflictBlock;
}


int
ConflictList::findConflictBlockAt( int nLineIndex )
{
  for ( Conflicts::iterator it = m_conflicts.begin(); it != m_conflicts.end(); ++it )
  {
    ConflictBlock *pBlock = *it;
    if ( pBlock->startingLineNumber() > nLineIndex )
      break;

    if ( pBlock->isLineInside( nLineIndex ) )
      return it - m_conflicts.begin();
  }
  return -1;
}


void 
ConflictList::addConflict( ConflictBlock *pConflict )
{
  m_conflicts.push_back( pConflict );
}


ConflictBlock *
ConflictList::conflictAt( int nIndex )
{
  return m_conflicts.at( nIndex );
}


int 
ConflictList::conflictCount()
{
  return m_conflicts.size();
}


void 
ConflictList::insertedLineAt( int nLineIndex )
{
  int nBlockIndex = findConflictBlockAt( nLineIndex-1 );
  if ( nBlockIndex < 0 )
    return;

  m_conflicts.at(nBlockIndex)->increaseSizeBy( 1 );

  for ( int nIndex = nBlockIndex+1; nIndex < m_conflicts.size(); ++nIndex )
    m_conflicts.at( nIndex )->moveBlockBy( 1 );
}


void 
ConflictList::removedLinesAt( int nLineIndex, 
                              int nNumberOfLines )
{
  int nBlockIndex  = findConflictBlockAt( nLineIndex );
  if ( nBlockIndex < 0 )
    return;

  m_conflicts.at(nBlockIndex)->increaseSizeBy( -nNumberOfLines );

  for (int nIndex = nBlockIndex+1; nIndex < m_conflicts.size(); ++nIndex )
    m_conflicts.at(nIndex)->moveBlockBy( -nNumberOfLines );
}
