// //////////////////////////////////////////////////////////////////////////
// Header file ConflictList.h for class ConflictList
// (c)Copyright 2000, Baptiste Lepilleur.
// Created: 2001/06/17
// //////////////////////////////////////////////////////////////////////////
#ifndef CONFLICTLIST_H
#define CONFLICTLIST_H

#include <vector>
class ConflictBlock;


/*! \class ConflictList
 * \brief This class represents a list of conflicts.
 */
class ConflictList
{
public:
  /*! Constructs a ConflictList object.
   */
  ConflictList();

  /*! Destructor.
   */
  virtual ~ConflictList();

  void addConflict( ConflictBlock *pConflict );

  ConflictBlock *conflictAt( int nIndex );

  int conflictCount();

  int conflictBlockStatusFor( int nLineIndex,
                              int nSelectedConflictIndex );

  void insertedLineAt( int nLineIndex );

  void removedLinesAt( int nLineIndex, 
                       int nNumberOfLines );

private:
  /// Prevents the use of the copy constructor.
  ConflictList( const ConflictList &copy );

  /// Prevents the use of the copy operator.
  void operator =( const ConflictList &copy );

  /** Finds the index of the specified block.
   * @return Zero based Index of the block, -1 if none.
   */
  int findConflictBlockAt( int nLineIndex );

private:
  typedef std::vector<ConflictBlock *> Conflicts;
  Conflicts m_conflicts;
};



// Inlines methods for ConflictList:
// ---------------------------------



#endif  // CONFLICTLIST_H
