// //////////////////////////////////////////////////////////////////////////
// Header file ConflictTextBuilder.h for class ConflictTextBuilder
// (c)Copyright 2000, Baptiste Lepilleur.
// Created: 2001/06/16
// //////////////////////////////////////////////////////////////////////////
#ifndef CONFLICTTEXTBUILDER_H
#define CONFLICTTEXTBUILDER_H

#include <string>


/*! \class ConflictTextBuilder
 * \brief This class represents an abstract conflict text buffer.
 */
class ConflictTextBuilder
{
public:
  /*! Constructs a ConflictTextBuilder object.
   */
  ConflictTextBuilder();

  /*! Destructor.
   */
  virtual ~ConflictTextBuilder();

  virtual void setEndOfLineDos() =0;
  virtual void setEndOfLineMac() =0;
  virtual void setEndOfLineUnix() =0;
  virtual void appendLine( std::string line ) =0;
  virtual void beginConflictBlock() =0;
  virtual void endConflictBlock() =0;

  enum EndOfLineType
  {
    eolDos =1,    // 0x0d 0x0a
    eolUnix,      // 0x0a
    eolMac,       // 0x0d
    eolDefault = eolDos
  };

private:
  /// Prevents the use of the copy constructor.
  ConflictTextBuilder( const ConflictTextBuilder &copy );

  /// Prevents the use of the copy operator.
  void operator =( const ConflictTextBuilder &copy );

private:
};



// Inlines methods for ConflictTextBuilder:
// -----------------------------------------------



#endif  // CONFLICTTEXTBUILDER_H
