// //////////////////////////////////////////////////////////////////////////
// Header file LineReader.h for class LineReader
// (c)Copyright 2000, Baptiste Lepilleur.
// Created: 2001/06/16
// //////////////////////////////////////////////////////////////////////////
#ifndef LINEREADER_H
#define LINEREADER_H

#include <stdexcept>
class ConflictTextBuilder;


/*! \class LineReader
 * \brief This class represents an abstract line reader.
 */
class LineReader
{
public:
  static LineReader *makeLineReaderFor( CFile *pFile );

  /*! Constructs a LineReader object.
   */
  LineReader( CFile *pFile );

  /*! Destructor.
   */
  virtual ~LineReader();

  virtual void setEndOfLineType( ConflictTextBuilder *pBuffer ) =0;
  virtual std::string readLine() =0;

  class NoMoreLine : public std::logic_error
  {
  public:
    NoMoreLine() : logic_error( "No more line to read" )
    {
    }
  };

private:
  /// Prevents the use of the copy constructor.
  LineReader( const LineReader &copy );

  /// Prevents the use of the copy operator.
  void operator =( const LineReader &copy );

protected:
  CFile m_file;
  CArchive m_archive;
};



// Inlines methods for LineReader:
// -------------------------------



#endif  // LINEREADER_H
